import 'dart:convert';

import 'package:core_encode/src/utils/entity.dart';
import 'package:injectable/injectable.dart';
import 'package:http/http.dart';


abstract class HttpServiceBase {
  final Client http;

  HttpServiceBase({required this.http});

  Future<Map> get({required String url, Map<String, String>? headers});

  Future<Map> post(
      {required String url,
      required Entity data,
      Map<String, String>? headers});
}

@Injectable(as: HttpServiceBase)
class HttpService extends HttpServiceBase {
  HttpService({required super.http});

  @override
  Future<Map> get({required String url, Map<String, String>? headers}) async {
    var result = await http.get(Uri.parse(url), headers: headers);
    return jsonDecode(result.body);
  }

  @override
  Future<Map> post({
    required String url,
    required Entity data,
    Map<String, String>? headers,
  }) async {
    var isUrlEncode = checkIfIsUrlEncoded(headers ?? {});
    var result = await http.post(
      Uri.parse(url),
      headers: headers,
      body: isUrlEncode ? data.toJson() : jsonEncode(data.toJson()),
    );
    return jsonDecode(result.body);
  }

  bool checkIfIsUrlEncoded(Map<String, String> headers) {
    var isUrlEncode = false;
    headers.forEach((key, value) {
      if (key == 'Content-Type' &&
          value == 'application/x-www-form-urlencoded') {
        isUrlEncode = true;
      }
    });
    return isUrlEncode;
  }
}
